@@ -20,19 +20,19 @@
  *
  *****************************************************************************/
 
-#ifndef MAPNIK_IMAGE_FILITER_GRAMMAR_HPP
-#define MAPNIK_IMAGE_FILITER_GRAMMAR_HPP
+#ifndef MAPNIK_IMAGE_FILTER_GRAMMAR_HPP
+#define MAPNIK_IMAGE_FILTER_GRAMMAR_HPP
+
+// mapnik
+#include <mapnik/config.hpp>
+#include <mapnik/css_color_grammar.hpp>
+#include <mapnik/color.hpp>
 
 #pragma GCC diagnostic push
 #include <mapnik/warning_ignore.hpp>
 #include <boost/spirit/include/qi.hpp>
-#include <boost/fusion/include/adapt_struct.hpp>
 #pragma GCC diagnostic pop
 
-// mapnik
-#include <mapnik/css_color_grammar.hpp>
-#include <mapnik/color.hpp>
-
 // stl
 #include <cmath>
 
@@ -43,26 +43,11 @@ struct color_stop;
 struct colorize_alpha;
 }
 
-}
-
-BOOST_FUSION_ADAPT_STRUCT(
-    mapnik::filter::color_stop,
-    (mapnik::color, color )
-    (double, offset)
-)
-
-namespace mapnik {
-
 namespace qi = boost::spirit::qi;
 
 struct percent_offset_impl
 {
-    template <typename T>
-    struct result
-    {
-        using type = double;
-    };
-
+    using result_type = double;
     double operator() (double val) const
     {
         double result = std::abs(val/100.0);
@@ -76,21 +61,27 @@ template <typename Iterator, typename ContType>
 struct image_filter_grammar :
         qi::grammar<Iterator, ContType(), qi::ascii::space_type>
 {
+    using alternative_type = qi::rule<Iterator, ContType(), qi::ascii::space_type>;
+
     image_filter_grammar();
+
     qi::rule<Iterator, ContType(), qi::ascii::space_type> start;
-    qi::rule<Iterator, ContType(), qi::ascii::space_type> filter;
-    qi::rule<Iterator, qi::locals<int,int>, void(ContType&), qi::ascii::space_type> agg_blur_filter;
-    qi::rule<Iterator, qi::locals<double,double,double,double,double,double,double,double>,
-             void(ContType&), qi::ascii::space_type> scale_hsla_filter;
-    qi::rule<Iterator, qi::locals<mapnik::filter::colorize_alpha, mapnik::filter::color_stop>, void(ContType&), qi::ascii::space_type> colorize_alpha_filter;
+    qi::rule<Iterator, ContType(), qi::ascii::space_type,
+                                   qi::locals<alternative_type*>> filter;
     qi::rule<Iterator, qi::ascii::space_type> no_args;
+    qi::symbols<char, alternative_type*> alternatives;
     qi::uint_parser< unsigned, 10, 1, 3 > radius_;
     css_color_grammar<Iterator> css_color_;
-    qi::rule<Iterator,void(mapnik::filter::color_stop &),qi::ascii::space_type> color_stop_offset;
-    phoenix::function<percent_offset_impl> percent_offset;
-    qi::rule<Iterator, qi::locals<color>, void(ContType&), qi::ascii::space_type> color_to_alpha_filter;
+    qi::rule<Iterator, filter::color_stop(), qi::ascii::space_type> color_stop_;
+    qi::rule<Iterator, double(), qi::ascii::space_type> color_stop_offset;
+
+private:
+    alternative_type & add(std::string const& symbol);
+    static constexpr unsigned max_alternatives = 16;
+    unsigned num_alternatives = 0;
+    alternative_type alternative_storage[max_alternatives];
 };
 
-}
+} // namespace mapnik
 
-#endif // MAPNIK_IMAGE_FILITER_PARSER_HPP
+#endif // MAPNIK_IMAGE_FILTER_GRAMMAR_HPP
