@@ -29,7 +29,7 @@ if env.get('BOOST_LIB_VERSION_FROM_HEADER'):
         can_build = True
 
 if not can_build:
-    print 'WARNING: skipping building the optional CSV datasource plugin which requires boost >= 1.56'
+    print ('WARNING: skipping building the optional CSV datasource plugin which requires boost >= 1.56')
 else:
     Import ('plugin_base')
 
