@@ -1,9 +1,8 @@
+
 #include "catch.hpp"
 
 #include <iostream>
 #include <mapnik/symbolizer.hpp>
-#include <vector>
-#include <algorithm>
 
 using namespace mapnik;
 
